/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-conf-8.c,v 1.10 2014/02/12 16:51:51 ca Exp $")

#include "sm/string.h"
#include "sm/net.h"
#include "sm/io.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#if MTA_USE_TLS
#define SM_TLSREQCNFDEF 1
#define SM_SSCNFDEF 1
#define TLSCNF_FLAGS_ISSET 1
#include "sm/tlscnf.h"
#include "sm/tlsreqcnf.h"
#include "sm/tlsreqcnfs.h"
#include <stdio.h>

/*
**  Just a test program to play around with libconf.
*/

/* session features */
struct ss_srv_feat_S {
	char		*sssf_name;
	uint32_t	 sssf_max_ta;
	tls_cnf_T	 sssf_tls;
#if SM_LIBCONF_ISSET
	uchar		 sssf_isset_name;
	uchar		 sssf_max_ta_isset;
#endif
};
typedef struct ss_srv_feat_S ss_srv_feat_T, *ss_srv_feat_P;

/* array of session features */
struct ss_srv_feats_S {
	ss_srv_feat_T  *sssfs_feat;
	uint            sssfs_n;
};
typedef struct ss_srv_feats_S ss_srv_feats_T, *ss_srv_feats_P;

struct ss_cnf_S
{
	tlsreq_cnf_T	 ssse_tlsreq_cnf;
	char			*ssse_feat_name;
	tls_cnf_T		 ssse_cnf_tls;
	ss_srv_feats_T	 ss_cnf_srv_feats;
};
typedef struct ss_cnf_S		ss_cnf_T, *ss_cnf_P;

static sm_conf_definition_T
ssse_tlsreq_defs[] = {

TLSREQ_DEFS(tlsreq_cnf_T, tlsreqcnf),

CONF_SENTINEL
};

#define SSC_TLS_REL_VRFY	0x01
#define SSC_TLS_RQST_CERT	0x04

sm_conf_definition_T const
ss_tls_flag_names[]
=
{
{ SM_CONF_DEF_MAGIC,
	"allow_relaying_if_verified",		sm_conf_type_choice_value,
	SSC_TLS_REL_VRFY,
	0, NULL, 0, NULL, NULL, NULL,
	"allow relaying if client cert has been verified"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC,
	"request_cert",	sm_conf_type_choice_value,
	SSC_TLS_RQST_CERT,
	0, NULL, 0, NULL, NULL, NULL,
	"request cert"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
}
;

sm_conf_definition_T
tls_defs[]
=
{
{ SM_CONF_DEF_MAGIC, "cert_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_cert),	0,
	"smcert.pem",
	SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"File with certificate in PEM format"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_cert_isset))
	SM_LC_SET_MAGIC(0) }
,{ SM_CONF_DEF_MAGIC, "key_file",	sm_conf_type_string,
	offsetof(tls_cnf_T, tlscnf_key),	0,
	"smkey.pem",
	SM_CONF_FLAG_CHK0|SM_CONF_FLAG_CHK1,	NULL,	NULL,	NULL,
	"File with private key for certificate in PEM format"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_key_isset))
	SM_LC_SET_MAGIC(0) }

,{ SM_CONF_DEF_MAGIC, "flags",		sm_conf_type_choice,
	offsetof(tls_cnf_T, tlscnf_cflags),	sizeof(uint32_t),
	NULL /* SM_XSTR(SSC_TLS_RQST_CERT) * how to set the default? */,
	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR
		|SM_CONF_FLAG_SECTION_DEFAULT_FROM_ANONYMOUS|SM_CONF_FLAG_PLURAL|SM_CONF_FLAG_NUMERIC|SM_CONF_FLAG_CHK1,
	ss_tls_flag_names,
	NULL,	NULL,
	"flags to influence behavior related to STARTTLS [see documentation]"
	SM_LC_ISSET(offsetof(tls_cnf_T, tlscnf_flags_isset))
	SM_LC_SET_MAGIC(0) }

,CONF_SENTINEL
};


sm_conf_definition_T ss_srv_feat_defs[] = {
{ SM_CONF_DEF_MAGIC, "name",		sm_conf_type_section_title,
	offsetof(ss_srv_feat_T, sssf_name),	0,
	NULL,	0 /*SM_CONF_FLAG_STRICTLY_REQUIRED*/,
	NULL, NULL, NULL, NULL
	SM_LC_ISSET(offsetof(ss_srv_feat_T, sssf_isset_name))
	},
{ SM_CONF_DEF_MAGIC, "max_ta",		sm_conf_type_u32,
	offsetof(ss_srv_feat_T, sssf_max_ta),	sizeof(uint32_t),
	NULL, SM_CONF_FLAG_DEFAULT_FROM_ENVIRONMENT,
	NULL, NULL,	NULL,
	"max ta"
	SM_LC_ISSET(offsetof(ss_srv_feat_T, sssf_max_ta_isset))
	},
{ SM_CONF_DEF_MAGIC, "tls", sm_conf_type_section,
	offsetof(ss_srv_feat_T, sssf_tls), sizeof(ss_srv_feat_T), NULL,
	SM_CONF_FLAG_SECTION_DEFAULT_FROM_ANONYMOUS|SM_CONF_FLAG_SET1,
	tls_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

sm_conf_definition_T ss_srv_feat_definitions[] = {
{ SM_CONF_DEF_MAGIC, "",	sm_conf_type_section,
	offsetof(ss_cnf_T, ss_cnf_srv_feats.sssfs_feat), sizeof(ss_srv_feat_T),
	NULL, 0,
	ss_srv_feat_defs,
	NULL, NULL, "session features" },
{ SM_CONF_DEF_MAGIC, "n",	sm_conf_type_array_n,
	offsetof(ss_cnf_T, ss_cnf_srv_feats.sssfs_n), sizeof(uint),
	NULL, 0,
	ss_srv_feat_defs,
	NULL, NULL,
	"number of session features (implicit)" },
CONF_SENTINEL
};


static sm_conf_definition_T
ssse_defs[] = {
{ SM_CONF_DEF_MAGIC, "tls_requirements", sm_conf_type_section,
	0, sizeof(tlsreq_cnf_T), NULL,
	SM_CONF_FLAG_KEEP_DEFAULT,
	ssse_tlsreq_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "tls", sm_conf_type_section,
	0, sizeof(tls_cnf_T), NULL,
	0,
	tls_defs,
	NULL, NULL, NULL
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "session_features",		sm_conf_type_array,
	0,	/* offset */
	0,	/* size (# of elements) */
	NULL,	/* default */
	SM_CONF_FLAG_MULTIPLE,
	ss_srv_feat_definitions,
	NULL,  NULL,
	"session feature declarations"	},

{ SM_CONF_DEF_MAGIC, "session_feature",	sm_conf_type_string,
	offsetof(ss_cnf_T, ssse_feat_name),	0,
	NULL,
	0,	NULL,	NULL,	NULL,
	"name of session feature"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};


static ss_cnf_T ss_cnf;

static int
process(char const *confname, FILE *fp, bool prt)
{
	sm_conf_T *smc;
	int err;
	char buf[SM_CONF_ERROR_BUFFER_SIZE];
	char const *e = NULL;

	if (((smc = sm_conf_new(confname ? confname : "*stdin*"))) == NULL) {
		sm_io_fprintf(smioout, "error -- sm_conf_new() returns NULL!\n");
		return 1;
	}
	if ((err = sm_conf_read_FILE(smc, confname, fp)) != 0) {
		sm_io_fprintf(smioout, "%s: %s\n",
			confname ? confname : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(smc, e)) != NULL)
			sm_io_fprintf(smioout, "%s\n", e);

		sm_conf_destroy(smc);
		return 2;
	}

	err = sm_conf_scan(smc, ssse_defs,
		SM_CONF_FLAG_ALLOW_ANY|SM_CONF_FLAG_KEEP_DEFAULT, &ss_cnf);
	if (err != 0) {
		sm_io_fprintf(smioout, "(while scanning) %s: %s\n",
			confname ? confname : "*stdin*",
			sm_conf_strerror(err, buf, sizeof buf));

		while ((e = sm_conf_syntax_error(smc, e)) != NULL)
			sm_io_fprintf(smioout, "%s\n", e);

		sm_conf_destroy(smc);
		return 3;
	}

	if (prt) {
		sm_str_P text;
		int n, u;

		text = sm_str_new(NULL, 256, 8192);
		sm_io_fprintf(smioout, "prt_conf1:\n");
		(void) sm_conf_prt_cnfs(ssse_defs, &ss_cnf, 0, smioout, 0, NULL,
				text, 0);

		n = ss_cnf.ss_cnf_srv_feats.sssfs_n;
		sm_io_fprintf(smioout, "n=%u\n", n);
		for (u = 0; u < n; u++) {
			sm_io_fprintf(smioout, "name[%u]=%s\n", u,
				ss_cnf.ss_cnf_srv_feats.sssfs_feat[u].sssf_name);
			sm_io_fprintf(smioout, "tls_flags[%u]=0x%08x (%x)\n", u
				, ss_cnf.ss_cnf_srv_feats.sssfs_feat[u].sssf_tls.tlscnf_cflags
				, ss_cnf.ss_cnf_srv_feats.sssfs_feat[u].sssf_tls.tlscnf_flags_isset
				);
		}

		sm_io_flush(smioout);
	}

	return 0;
}

int
main(int ac, char **av)
{
	int ai, c, ret;
	bool prt, done;
	char *option, *opt;

	done = false;
	prt = false;
	option = NULL;
	opt = NULL;
	while ((c = getopt(ac, av, "df:O:o:s")) != -1) {
		switch (c) {
		  case 'd':
			sm_conf_prt_dflt(ssse_defs, SMC_FLD_FLAGS, smioout);
			sm_io_flush(smioout);
			return 0;
		  case 'f':
			ret = process(optarg, NULL, prt);
			if (ret != 0)
				return ret;
			done = true;
			break;
		  case 'O':
			opt = strdup(optarg);
			break;
		  case 'o':
			option = strdup(optarg);
			break;
		  case 's':
			prt = true;
			break;
		}
	}
	ac -= optind;
	av += optind;
	if (done) {
		if (prt) {
			sm_io_fprintf(smioout, "prt_conf:\n");
			(void) sm_conf_prt_cnfs(ssse_defs, &ss_cnf, 0,
					smioout, 0, NULL, NULL, 0);
			sm_io_flush(smioout);
		}
		return 0;
	}

	if (ac == 0)
		return process("*stdin*", stdin, prt);

	for (ai = 0; ai < ac; ai++) {
		int ret;

		ret = process(av[ai], NULL, prt);
		if (ret != 0)
			return ret;
	}
	return 0;
}
#else /* MTA_USE_TLS */
int
main(int ac, char **av)
{
	sm_io_fprintf(smioerr, "MTA_USE_TLS not set\n");
	return 0;
}
#endif /* MTA_USE_TLS */
